@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.62 $
+  $Revision: 1.75 $
 
   DBupdate 
 
@@ -34,7 +34,7 @@
 
 
 
-
+#include <config.h>
 #include "dbupdate.h"
 #include "UP_extrnl_syntax.h"
 #include "UP_subject.h"
@@ -49,7 +49,7 @@
 #include "mail_parser.h"
 #include "process.h"
 
-#ifdef HAVE_PARAM_H
+#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
 
@@ -65,6 +65,10 @@ int reading_from_mail = 0;
 /* are we processing networkupdate? */
 int networkupdate = 0;
 
+/* IP of the networkupdate client */
+char * netupdclientIP = NULL;
+
+
 /* sender of the mail, in case of a mail update */
 char *update_mail_sender = NULL;
 char *update_mail_subject = NULL;
@@ -78,6 +82,7 @@ char *lockdir = NULL;
 char *mailcmd = NULL;
 char *notitxt = NULL;
 char *notimailtxt = NULL;
+char *notinetworktxt = NULL;
 char *fwtxt   = NULL;
 char *fwmailtxt = NULL;
 char *mtfwheader = NULL; 
@@ -85,12 +90,14 @@ char *mtfwtxt = NULL;
 char *mailtxt = NULL;
 char *acksig = NULL;
 char *defmail = NULL;
+char *updlog = NULL;
 char *notiflog = NULL;
 char *crosslog = NULL;
 char *acklog = NULL;
 char *forwlog = NULL;
 char *humailbox = NULL;
 char *autobox = NULL;
+char *copyright_notice = NULL;
 char *overridecryptedpw = NULL;
 char *country = NULL;
 char *countries[400];
@@ -123,6 +130,9 @@ char *DBname = NULL;
 char *DBpasswd = NULL;
 /* end of config variables */
 
+char * fingerprint = NULL;
+char * keyowner = NULL;
+
 /* result of subject line processing is kept in the following struct. */
 up_subject_struct subject_result;
 
@@ -289,7 +299,7 @@ char * import_key(char *obj){
   char * temp, * temp2;
    
   tempfile = (char *)malloc(strlen(tmpdir) + strlen("tmp-key.") + 32);
-  sprintf(tempfile, "%s/tmp-key.%i", tmpdir, pid /*getpid()*/);
+  sprintf(tempfile, "%s/tmp-key.%i", tmpdir, pid );
 
   /* now we must write certif attribute(s) of this key-certif into the tempfile */
   /* get the certif first */
@@ -317,7 +327,6 @@ char * import_key(char *obj){
       }
     }
     g_strfreev(lines);
-    //fprintf(key_file, "%s\n", (char *)next->data);
   }
   fclose(key_file);
   g_slist_free(certiflist);
@@ -326,12 +335,17 @@ char * import_key(char *obj){
   strcpy(iKO.keyRing, pgp_public_key_ring);
   PA_ImportKey(&iKO);
 
-  printf("importKeyObj status:\n");
+  if(tracing){
+    printf("importKeyObj status:\n");
     
-  printf("isValid: %d\n", iKO.rc);
-  printf("keyID: %08lX\n", iKO.keyID);
+    printf("isValid: %d\n", iKO.rc);
+    printf("keyID: %08lX\n", iKO.keyID);
+  }
   snprintf(keyID, 9, "%08lX", iKO.keyID);
-  printf("keyID: [%s]\n", keyID);
+  
+  if(tracing){
+    printf("keyID: [%s]\n", keyID);
+  }
   
   unlink(tempfile);
   free(tempfile);
@@ -341,12 +355,18 @@ char * import_key(char *obj){
   key_cert_attr = strdup((char *)templist->data);
   g_slist_free(templist);
 
-  printf("key_cert_attr: [%s]\n", key_cert_attr);
+  if(tracing){
+    printf("key_cert_attr: [%s]\n", key_cert_attr);
+  }
   obj_keyID = strdup(key_cert_attr + strlen("PGPKEY-"));
-  printf("obj_keyID: [%s]\n", obj_keyID);
-  if(iKO.rc == iKO_OK && (strcmp(obj_keyID, keyID) == 0)){/* if PA_ImportKey returned OK 
+  if(tracing){
+    printf("obj_keyID: [%s]\n", obj_keyID);
+  }
+  if(iKO.rc == iKO_OK && (strcasecmp(obj_keyID, keyID) == 0)){/* if PA_ImportKey returned OK 
                                                             and the real keyID is equal to the
                                                             keyID in the 'key-cert' attribute */
+    fingerprint = strdup(iKO.fingerPrint); 
+    keyowner    = strdup(iKO.keyOwner);                                                           
     return NULL;
   }else{/* if PA_ImportKey returned not OK or obj_keyID, keyID didn't match */
     if(iKO.rc != iKO_OK){
@@ -376,6 +396,65 @@ char * import_key(char *obj){
 
 
 
+
+
+/* Gets the keyowner and fingerprint of a PGP key 
+   from the keycert object. The keycert object must already
+   be in the database (thus, in the keyring) 
+   The fingerprint and keyowner will be saved in global variables */
+   
+void * get_keyowner_fingerpr(char *obj){
+
+  GSList * templist = NULL;
+  char * obj_keyID = NULL;
+  struct ImportKeyObject iKO;
+  u32 keyID;
+  char * key_cert_attr;
+   
+  
+  templist = get_attr_list(obj, "key-cert");
+  key_cert_attr = strdup((char *)templist->data);
+  g_slist_free(templist);
+
+  if(tracing){
+    printf("get_keyowner_fingerpr: key_cert_attr is [%s]\n", key_cert_attr);
+  } 
+  obj_keyID = strdup(key_cert_attr + strlen("PGPKEY-"));
+  if(tracing){
+    printf("get_keyowner_fingerpr: obj_keyID is [%s]\n", obj_keyID);
+  }
+ 
+  sscanf(obj_keyID, "%8X", &keyID); 
+  if(tracing){
+    printf("get_keyowner_fingerpr: keyID is [%s]\n", keyID);
+  }
+  
+  /* set appropriate fields of iKO */ 
+  iKO.keyID = keyID;
+  strcpy(iKO.keyRing, pgp_public_key_ring);
+
+  GetFingerPrint(&iKO);
+  fingerprint = strdup(iKO.fingerPrint);
+  
+  GetKeyOwner(&iKO);
+  keyowner = strdup(iKO.keyOwner); 
+
+
+  if(tracing){
+    if(fingerprint != NULL){
+      printf("get_keyowner_fingerpr: fingerprint is [%s]\n", fingerprint);
+    }
+    if(keyowner){
+      printf("get_keyowner_fingerpr: keyowner is [%s]\n", keyowner);
+    }
+  }
+}
+
+
+
+
+
+
 /* Checks the object's syntax, retrives the old version of it from the db, 
    and checks auth2. If everything is OK, then sends it to RIPdb, where referential
    integrity is checked, and the object is really committed to the db.
@@ -399,15 +478,19 @@ int process_object(char * incoming, credentials_struct credentials,
     char * old_version = NULL;
     o = new Object;
     int result = 0;
-    int result_from_RIPupd = 0;
+    //int result_from_RIPupd = 0;
+    up_ripupd_result_struct * result_from_RIPupd;
     char * result_from_import_key = NULL;
     char * result_from_delete_key = NULL;
     char * auto_nic = NULL;
     char * changed_obj = NULL;
     char * obj_with_AUTO_NIC_hdl;
     char * assigned_NIC;
+    char * formatted_object;
     char * type;
     char * arg;
+    char * arg2;
+    char * generated_object;
     external_syntax_struct * external_syntax_results;
 
 
@@ -446,7 +529,9 @@ int process_object(char * incoming, credentials_struct credentials,
                         o->type->getName(), get_search_key(o, o->type->getName(), arg), arg);
           return UP_NSO; /* no such object */
         }else {/* the object is in the db */
-          if(identical(old_version, arg)){/* if the old & new versions are identical */
+          if(identical(old_version, arg) /* if the old & new versions are identical */
+             || (strcmp(o->type->getName(), "key-cert") == 0)  /* or it is a key-cert object  */
+             || (strcmp(o->type->getName(), "inet6num") == 0)){/* or it is an inet6num object */
             result = check_auth(NULL, old_version, o->type->getName(), credentials);
             if(result == UP_AUTH_OK){ 
               if(tracing) {
@@ -460,15 +545,17 @@ int process_object(char * incoming, credentials_struct credentials,
               /* if there was no problem with key deletion from the key-ring */
               if(result_from_delete_key == NULL){
                 result_from_RIPupd = send_object_db(arg, NULL, "DEL");
-                if(result_from_RIPupd == 0){
+                if(result_from_RIPupd->result == 0){
                   AK_add_to_ack(ack_file_name, "\nDelete OK: [%s] %s\n", 
                                 o->type->getName(), get_search_key(o, o->type->getName(), arg));
-                  NT_write_all_ntfs(arg, NULL, tmpdir, ntfy_hash, forw_hash, cross_hash, credentials.from);
+                  NT_write_all_ntfs(old_version, NULL, NULL, tmpdir, ntfy_hash, forw_hash, cross_hash, 
+                                    credentials.from);
                 }else{
-                  AK_add_to_ack(ack_file_name, "\nDelete FAILED: [%s] %s\nReferential integrity failure\n",
-                                o->type->getName(), get_search_key(o, o->type->getName(), arg));
+                  AK_add_to_ack(ack_file_name, "\nDelete FAILED: [%s] %s\n%s\n",
+                                o->type->getName(), get_search_key(o, o->type->getName(), arg),
+                                result_from_RIPupd->error_str);
                 }
-                result_from_RIPupd = 0;
+                result_from_RIPupd->result = 0;
               }else{
                  AK_add_to_ack(ack_file_name, "\nDelete FAILED: [%s] %s\n%s\n",
                                 o->type->getName(), get_search_key(o, o->type->getName(), arg), result_from_delete_key);
@@ -478,13 +565,14 @@ int process_object(char * incoming, credentials_struct credentials,
                 printf("TRACING: Auth failed\n");
               }
 
-              AK_add_to_ack(ack_file_name, "\nDelete FAILED: [%s] %s\nAuth failed\n",
-                            o->type->getName(), get_search_key(o, o->type->getName(), arg));
+              AK_add_to_ack(ack_file_name, "\nDelete FAILED: [%s] %s\nAuthorisation failed, request forwarded to maintainer.\n%s\n",
+                            o->type->getName(), get_search_key(o, o->type->getName(), arg), arg);
               NT_write_all_frwds(arg, NULL, tmpdir, ntfy_hash, forw_hash, cross_hash, credentials.from);
               return UP_AUF; /* Auth failed */
             } 
           }else{/* the new & old versions do not match */
-            AK_add_to_ack(ack_file_name, "\nDelete FAILED: new & old versions do not match\n");
+            AK_add_to_ack(ack_file_name, "\nDelete FAILED: [%s] %s\nnew & old versions do not match\n%s\n",
+                 o->type->getName(), get_search_key(o, o->type->getName(), arg), arg);
             return UP_NOM; /* new & old versions do not match */
           }
         }
@@ -510,15 +598,21 @@ int process_object(char * incoming, credentials_struct credentials,
               }
               assigned_NIC = (char *)malloc(128); /* this should be enough for a NIC hdl */
               result_from_RIPupd = send_object_db(obj_with_AUTO_NIC_hdl, assigned_NIC, "ADD");
-              if(result_from_RIPupd == 0){
+              if(result_from_RIPupd->result == 0){
                 AK_add_to_ack(ack_file_name, "\nNew OK: [%s] %s\n", 
                               o->type->getName(), assigned_NIC);
-                NT_write_all_ntfs(NULL, arg, tmpdir, ntfy_hash, forw_hash, cross_hash, credentials.from);
+                /* replace the AUTO nic hdl with the assigned one (for reporting purposes, in the notif mesg) */
+                formatted_object = UP_put_assigned_NIC(external_syntax_results->new_obj, assigned_NIC);
+                formatted_object = delete_override(formatted_object);
+                arg2 = delete_override(arg);
+                NT_write_all_ntfs(NULL, arg2, formatted_object, tmpdir, ntfy_hash, forw_hash, cross_hash, 
+                                  credentials.from);
               }else{
-                AK_add_to_ack(ack_file_name, "\nNew FAILED: [%s] %s\nReferential integrity failure\n",
-                              o->type->getName(), arg);
+                AK_add_to_ack(ack_file_name, "\nNew FAILED: [%s]\n%s\n%s\n",
+                              o->type->getName(), arg,
+                              result_from_RIPupd->error_str);
               }
-              result_from_RIPupd = 0;
+              result_from_RIPupd->result = 0;
               if(tracing && assigned_NIC != NULL) {  
                 printf("TRACING: send_object_db returned [%s] as assigned NIC hdl\n", assigned_NIC);
               }
@@ -538,13 +632,14 @@ int process_object(char * incoming, credentials_struct credentials,
                 printf("TRACING: Auth failed\n");
               }
   
-              AK_add_to_ack(ack_file_name, "\nNew FAILED: [%s] %s\nAuth failed\n",
-                            o->type->getName(), get_search_key(o, o->type->getName(), arg));
-              NT_write_all_frwds(NULL, arg, tmpdir, ntfy_hash, forw_hash, cross_hash, credentials.from);
+              AK_add_to_ack(ack_file_name, "\nNew FAILED: [%s] %s\nAuthorisation failed, request forwarded to maintainer.\n%s\n",
+                            o->type->getName(), get_search_key(o, o->type->getName(), arg), arg);
+              arg2 = delete_override(arg);
+              NT_write_all_frwds(NULL, arg2, tmpdir, ntfy_hash, forw_hash, cross_hash, credentials.from);
               return UP_AUF; /* Auth failed */
             }
           }else{/* external syntax check failed */
-            AK_add_to_ack(ack_file_name, "\nUpdate FAILED: [%s] %s\n%s%s",
+            AK_add_to_ack(ack_file_name, "\nUpdate FAILED: [%s] %s\n%s%s\n",
                    o->type->getName(), get_search_key(o, o->type->getName(), arg), 
                    arg, external_syntax_results->error_str);
           }
@@ -565,25 +660,37 @@ int process_object(char * incoming, credentials_struct credentials,
                     if(tracing) {                                
                       printf("TRACING: Will send the obj to be updated\n");
                     }
+
+                    if(strcasecmp(o->type->getName(), "key-cert") == 0){
+                      get_keyowner_fingerpr(arg);
+                      generated_object = UP_generate_kc_attrs(o, arg);
+                      external_syntax_results->new_obj = generated_object;
+                    }
+                    
                     result_from_RIPupd = send_object_db(external_syntax_results->new_obj, NULL, "UPD");
-                    if(result_from_RIPupd == 0){
+                    if(result_from_RIPupd->result == 0){
                       AK_add_to_ack(ack_file_name, "\nUpdate OK: [%s] %s\n",
                                     o->type->getName(), get_search_key(o, o->type->getName(), arg));
-                      NT_write_all_ntfs(old_version, arg, tmpdir, ntfy_hash, forw_hash, cross_hash, credentials.from);
+                      arg2 = delete_override(arg);
+                      NT_write_all_ntfs(old_version, arg2, NULL, tmpdir, ntfy_hash, forw_hash, cross_hash, 
+                                        credentials.from);
                     }else{
-                      AK_add_to_ack(ack_file_name, "\nUpdate FAILED: [%s] %s\nReferential integrity failure\n",
-                                    o->type->getName(), get_search_key(o, o->type->getName(), arg));
+                      AK_add_to_ack(ack_file_name, "\nUpdate FAILED: [%s] %s\n%s\n",
+                      
+                                    o->type->getName(), get_search_key(o, o->type->getName(), arg),
+                                    result_from_RIPupd->error_str);
                     }
-                    result_from_RIPupd = 0;
+                    result_from_RIPupd->result = 0;
                   }else{
                     /* auth failed ! */
                     if(tracing) {
                       printf("TRACING: Auth failed\n");
                     }
       
-                    AK_add_to_ack(ack_file_name, "\nUpdate FAILED: [%s] %s\nAuth failed\n",
-                                  o->type->getName(), get_search_key(o, o->type->getName(), arg));
-                    NT_write_all_frwds(old_version, arg, tmpdir, ntfy_hash, forw_hash, cross_hash, credentials.from);
+                    AK_add_to_ack(ack_file_name, "\nUpdate FAILED: [%s] %s\nAuthorisation failed, request forwarded to maintainer.\n%s\n",
+                                  o->type->getName(), get_search_key(o, o->type->getName(), arg), arg);
+                    arg2 = delete_override(arg);
+                    NT_write_all_frwds(old_version, arg2, tmpdir, ntfy_hash, forw_hash, cross_hash, credentials.from);
                     return UP_AUF; /* Auth failed */
                   }
 
@@ -596,7 +703,7 @@ int process_object(char * incoming, credentials_struct credentials,
    
                 }
               }else{/* if there is an error in external syntax checks */
-                AK_add_to_ack(ack_file_name, "\nUpdate FAILED: [%s] %s\n%s%s",
+                AK_add_to_ack(ack_file_name, "\nUpdate FAILED: [%s] %s\n%s%s\n",
                      o->type->getName(), get_search_key(o, o->type->getName(), arg), 
                      arg, external_syntax_results->error_str);
               }
@@ -604,7 +711,7 @@ int process_object(char * incoming, credentials_struct credentials,
             }else{/* if the user enforced creation (using NEW keyword) in the subject line */
 
               AK_add_to_ack(ack_file_name, "\nUpdate FAILED: [%s] %s\n%s\n"
-                   "*ERROR*:  Object already exists\n",
+                   "***ERROR:  Object already exists\n",
                    o->type->getName(), get_search_key(o, o->type->getName(), arg), 
                    arg);
               
@@ -625,15 +732,27 @@ int process_object(char * incoming, credentials_struct credentials,
                   result_from_import_key = NULL;
                 }
                 if(result_from_import_key == NULL){/* no PGP problem */
-                  result_from_RIPupd = send_object_db(external_syntax_results->new_obj, NULL, "ADD");
-                  if(result_from_RIPupd == 0){/* if there was no problem */
-                    AK_add_to_ack(ack_file_name, "\nNew OK [%s] %s\n",
+                  if(strcmp(type, "key-cert") == 0){/* if the object is a key-cert object */
+
+                    generated_object = UP_generate_kc_attrs(o, arg);
+                    result_from_RIPupd = send_object_db(generated_object, NULL, "ADD");                     
+                  }else{
+                    
+                    result_from_RIPupd = send_object_db(external_syntax_results->new_obj, NULL, "ADD");
+
+                  }
+                  if(result_from_RIPupd->result == 0){/* if there was no problem */
+                    AK_add_to_ack(ack_file_name, "\nNew OK: [%s] %s\n",
                                   o->type->getName(), get_search_key(o, o->type->getName(), arg));
-                    NT_write_all_ntfs(NULL, arg, tmpdir, ntfy_hash, forw_hash, cross_hash, credentials.from);
+                    arg2 = delete_override(arg);
+                    NT_write_all_ntfs(NULL, arg2, NULL, tmpdir, ntfy_hash, forw_hash, cross_hash, 
+                                      credentials.from);
 
                   }else{
-                    AK_add_to_ack(ack_file_name, "\nNew FAILED: [%s] %s\nReferential integrity failure\n",
-                                  o->type->getName(), get_search_key(o, o->type->getName(), arg));
+                    //AK_add_to_ack(ack_file_name, "\nNew FAILED: [%s] %s\nReferential integrity failure\n",
+                    AK_add_to_ack(ack_file_name, "\nNew FAILED: [%s] %s\n%s\n",
+                                  o->type->getName(), get_search_key(o, o->type->getName(), arg),
+                                  result_from_RIPupd->error_str);
                   }
                   result_from_RIPupd = 0;
                 }else{/* there was a problem with PGP key import */
@@ -661,19 +780,32 @@ int process_object(char * incoming, credentials_struct credentials,
                 /* and forward this creation request to <HUMAILBOX> */
                 NT_forw_create_req(external_syntax_results->new_obj);
                 
+              }else if(result == UP_HOF){/* hierarchical authorisation failed */
+                if(tracing) {
+                  printf("TRACING: Auth failed\n");
+                }
+
+                AK_add_to_ack(ack_file_name, "\nNew FAILED: [%s] %s\nHierarchical authorisation failed, request forwarded to maintainer.\n%s\n",
+                              o->type->getName(), get_search_key(o, o->type->getName(), arg), arg);
+                arg2 = delete_override(arg);
+                NT_write_all_frwds(NULL, arg2, tmpdir, ntfy_hash, forw_hash, cross_hash, credentials.from);
+                return UP_AUF;
+                
+                
               }else{
                 /* auth failed ! */
                 if(tracing) {
                   printf("TRACING: Auth failed\n");
                 }
 
-                AK_add_to_ack(ack_file_name, "\nNew FAILED: [%s] %s\nAuth failed\n",
-                              o->type->getName(), get_search_key(o, o->type->getName(), arg));
-                NT_write_all_frwds(NULL, arg, tmpdir, ntfy_hash, forw_hash, cross_hash, credentials.from);
+                AK_add_to_ack(ack_file_name, "\nNew FAILED: [%s] %s\nAuthorisation failed, request forwarded to maintainer.\n%s\n",
+                              o->type->getName(), get_search_key(o, o->type->getName(), arg), arg);
+                arg2 = delete_override(arg);
+                NT_write_all_frwds(NULL, arg2, tmpdir, ntfy_hash, forw_hash, cross_hash, credentials.from);
                 return UP_AUF; /* Auth failed */
               }
             }else{
-              AK_add_to_ack(ack_file_name, "\nNew FAILED: [%s] %s\n%s%s",
+              AK_add_to_ack(ack_file_name, "\nNew FAILED: [%s] %s\n%s%s\n",
                               o->type->getName(), get_search_key(o, o->type->getName(), arg), 
                               arg, external_syntax_results->error_str);
             }
@@ -686,33 +818,89 @@ int process_object(char * incoming, credentials_struct credentials,
        if(tracing){   
          printf("TRACING: Object didn't parse\n");   
        }
-       AK_add_to_ack(ack_file_name, "\nUpdate FAILED: Syntax error in object\n");
+       /* if it is for deletion */
+       if(o->isDeleted){
+         /* here delete it */
+         old_version = get_old_version(arg);
+         if(old_version == NULL){ /* the object doesn't exist in the db! */
+            AK_add_to_ack(ack_file_name, "\nDelete FAILED: [%s] %s\nEntry not found\n\n%s\n", 
+                          o->type->getName(), get_search_key(o, o->type->getName(), arg), arg);
+            return UP_NSO; /* no such object */
+          }else {/* the object is in the db */
+            if(identical(old_version, arg)){/* if the old & new versions are identical */
+              result = check_auth(NULL, old_version, o->type->getName(), credentials);
+              if(result == UP_AUTH_OK){ 
+                if(tracing) {
+                  printf("TRACING: Will send the obj to be deleted\n");
+                }
+                if(strcmp(type, "key-cert") == 0){
+                  result_from_delete_key = delete_key(arg);
+                }else{
+                  result_from_delete_key = NULL;
+                }
+                /* if there was no problem with key deletion from the key-ring */
+                if(result_from_delete_key == NULL){
+                  result_from_RIPupd = send_object_db(arg, NULL, "DEL");
+                  if(result_from_RIPupd->result == 0){
+                    AK_add_to_ack(ack_file_name, "\nDelete OK: [%s] %s\n", 
+                                  o->type->getName(), get_search_key(o, o->type->getName(), arg));
+                    NT_write_all_ntfs(arg, NULL, NULL, tmpdir, ntfy_hash, forw_hash, cross_hash, 
+                                      credentials.from);
+                  }else{
+                    AK_add_to_ack(ack_file_name, "\nDelete FAILED: [%s] %s\n%s\n",
+                                  o->type->getName(), get_search_key(o, o->type->getName(), arg),
+                                  result_from_RIPupd->error_str);
+                  }
+                  result_from_RIPupd = 0;
+                }else{
+                   AK_add_to_ack(ack_file_name, "\nDelete FAILED: [%s] %s\n%s\n",
+                                  o->type->getName(), get_search_key(o, o->type->getName(), arg), result_from_delete_key);
+                }
+              }else{ /* auth failed */
+                if(tracing) {
+                  printf("TRACING: Auth failed\n");
+                }
+
+                AK_add_to_ack(ack_file_name, "\nDelete FAILED: [%s] %s\nAuthorisation failed, request forwarded to maintainer.\n%s\n",
+                              o->type->getName(), get_search_key(o, o->type->getName(), arg), arg);
+                NT_write_all_frwds(arg, NULL, tmpdir, ntfy_hash, forw_hash, cross_hash, credentials.from);
+                return UP_AUF; /* Auth failed */
+              } 
+            }else{/* the new & old versions do not match */
+              AK_add_to_ack(ack_file_name, "\nDelete FAILED: new & old versions do not match\n%s\n", arg);
+              return UP_NOM; /* new & old versions do not match */
+            }
+          }
+               
+       }else{/* syntax error AND not deletion */
+         AK_add_to_ack(ack_file_name, "\nUpdate FAILED: Syntax error in object\n");
        
-       if(o->attrs.head() != NULL){
-         for(attr = o->attrs.head(); attr; attr = o->attrs.next(attr)){
-           if(attr->len > 0){
-             value = (char*)malloc(attr->len);
-             strncpy(value, (char *)(arg+attr->offset) ,
-               attr->len - 1);
-             value[attr->len - 1] = '\0';
-             AK_add_to_ack(ack_file_name, "%s\n", value);
-             if(!attr->errors.empty()){
-               AK_add_to_ack_string(ack_file_name, attr->errors);
-             }
-             free(value);
-           }else{
-             if(!attr->errors.empty()){
-               AK_add_to_ack_string(ack_file_name, attr->errors);
+         if(o->attrs.head() != NULL){
+           for(attr = o->attrs.head(); attr; attr = o->attrs.next(attr)){
+             if(attr->len > 0){
+               value = (char*)malloc(attr->len);
+               strncpy(value, (char *)(arg+attr->offset) ,
+                 attr->len - 1);
+               value[attr->len - 1] = '\0';
+               AK_add_to_ack(ack_file_name, "%s\n", value);
+               if(!attr->errors.empty()){
+                 AK_add_to_ack_string(ack_file_name, attr->errors);
+               }
+               free(value);
+             }else{
+               if(!attr->errors.empty()){
+                 AK_add_to_ack_string(ack_file_name, attr->errors);
+               }
              }
            }
          }
-       }
-       if(o->has_error){
-         AK_add_to_ack_string(ack_file_name, o->errors);
-       }
-       AK_add_to_ack(ack_file_name, "\n");
+         if(o->has_error){
+           AK_add_to_ack_string(ack_file_name, o->errors);
+         }
+         AK_add_to_ack(ack_file_name, "\n");
        
-       return UP_NIY; /* XXX Not implemented yet */
+         return UP_SYN; /* syntax error */
+       }
     }
 }
 
@@ -731,22 +919,25 @@ GSList *next = NULL;
 int object_count = 0;
 char *object = NULL;
 char * line;
+char * lwrcase_line;
 int result = 0;
 struct VerifySignObject vSO, *pvSO;
 
 
   
-     line = (char *)malloc(1024);
+    line = (char *)malloc(1024);
 
-     if((input_file = fopen(filename, "r")) == NULL){
-         ER_perror(FAC_UP, UP_CANTOPEN, "Couldn't open the file %s: %s\n", filename, strerror(errno));
-         exit(1);  
-     }
+    if((input_file = fopen(filename, "r")) == NULL){
+        ER_perror(FAC_UP, UP_CANTOPEN, "Couldn't open the file %s: %s\n", filename, strerror(errno));
+        exit(1);  
+    }
 
   
     while(fgets(line, 1023, input_file) != NULL){
       /* first, if it is a pasword, save it, but do not regard it as an attrib */ 
-      if(strstr(line, "password:") == line){
+      lwrcase_line = strdup(line);
+      g_strdown(lwrcase_line);
+      if(strstr(lwrcase_line, "password:") == lwrcase_line){
         if(tracing){
           printf("TRACING: This is a password\n");
         }
@@ -754,6 +945,8 @@ struct VerifySignObject vSO, *pvSO;
                                       g_strstrip(strdup(line + strlen("password:"))));
         continue;
       }
+      free(lwrcase_line);
+      
       line = UP_remove_EOLs(line); /* remove '\n's and '\r' first */
       /* remove trailing white space */
       line = g_strchomp(line); 
@@ -1138,9 +1331,11 @@ void main(int argc, char **argv, char **envp){
   autobox = ca_get_autobox;
   overridecryptedpw = ca_get_overridecryptedpw;
   overridecryptedpw = g_strstrip(overridecryptedpw);
+  updlog = ca_get_updlog;
   acklog = ca_get_acklog;
   notiflog = ca_get_notiflog;
   notimailtxt = ca_get_notimailtxt;  
+  notinetworktxt = ca_get_notinetworktxt;
   forwlog = ca_get_forwlog;
   fwmailtxt = ca_get_fwmailtxt;
   mtfwheader = ca_get_mtfwheader;
@@ -1161,6 +1356,7 @@ void main(int argc, char **argv, char **envp){
   cno_explain_del = ca_get_cno_explain_del;
   cno_overlap_add = ca_get_cno_overlap_add;
   cno_overlap_del = ca_get_cno_overlap_del;
+  copyright_notice = ca_get_pw_resp_header;
   mheader = ca_get_mheader;
   pgp_public_key_ring = (char *)malloc(strlen(pgppath) + strlen("/pubring.gpg") + 2);
   sprintf(pgp_public_key_ring ,"%s/pubring.gpg", pgppath);
@@ -1245,7 +1441,7 @@ void main(int argc, char **argv, char **envp){
   pid = getpid(); 
 
   /* create the lock file and lock it */
-  lockfile = create_lock_file();
+  //lockfile = create_lock_file();
 
     
   /* initialize the parser */
@@ -1268,12 +1464,21 @@ void main(int argc, char **argv, char **envp){
       if(tracing){
         printf("TRACING: temp_upd_file is [%s]\n", temp_upd_file);
       }
+
+      /* first log the input in the upd log file */
+      UP_add_to_upd_log(input_file_name);
+ 
+      
       MM_store(input_file_name, temp_upd_file, 0);
       p = EP_ParseMail(input_file_name, tmpdir, pgp_public_key_ring, gpgcmd);
       
     }else{/* input_file_name == NULL */
       temp_upd_file = generate_upd_file();
       MM_store("-", temp_upd_file, 0);
+
+      /* first log the input in the upd log file */
+      UP_add_to_upd_log(temp_upd_file);
+      
       p = EP_ParseMail(temp_upd_file, tmpdir, pgp_public_key_ring, gpgcmd);
 
     }
@@ -1330,6 +1535,7 @@ void main(int argc, char **argv, char **envp){
     if(strlen(reply_to) > 0){
       to_address = (char *)realloc(to_address, strlen(reply_to) + 1);
       to_address = strcpy(to_address, reply_to);
+      to_address = find_email_address(to_address); /* so that we take only the email address */
     }
 
     if(p->message_id != NULL && p->message_id->field != NULL){
@@ -1451,6 +1657,10 @@ void main(int argc, char **argv, char **envp){
   }else{/* not reading from the mail message or from network */
     if(input_file_name != NULL){
 
+      /* first log the input in the upd log file */
+      UP_add_to_upd_log(input_file_name);
+
+      
       write_checkpoint(1);
       process_file(input_file_name, credentials, 
                   AUTO_NIC_hdl_hash, ack_file_name, 
@@ -1509,7 +1719,8 @@ void main(int argc, char **argv, char **envp){
   NT_delete_ntfy_list(cross_hash);
       
   /* remove the lock file */
-  remove_lock_file(lockfile);
+  //remove_lock_file(lockfile);
+  
  
   /* remove checkpoint file */
   remove_checkpoint();
