@@ -42,7 +42,7 @@ OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include <unistd.h>
 #include "ca_configFns.h"
 
-/* #define DEBUG */ /* Swich OFF Debugging. */
+/* #define DEBUG */	/* Swich OFF Debugging. */
 
 /*******************************************************
   * This file contains the definitions of functions     *
@@ -320,7 +320,7 @@ int no_source_defd;
 			/*
 			 * Check that the definition of the NRTM is complete.
 			 */
-			 if ( !( newMirrPtr->host && newMirrPtr->port && newMirrPtr->delay && newMirrPtr->protocolVer) )
+			 if ( !( newMirrPtr->host && newMirrPtr->port && newMirrPtr->protocolVer) && (newMirrPtr->delay < 0) )
 				{
 				fprintf(stderr, "Error: definition of near-real-time-mirror is incomplete.\n");
 				die;
