@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.28 $
+  $Revision: 1.29 $
 
   IP handling (ip). ip.c  - conversions between ascii and binary forms 
                             of IP addresses, prefixes and ranges.
@@ -474,7 +474,9 @@ IP_rang_t2b(ip_range_t *rangptr, char *rangstr, ip_exp_t expf)
 
     /* now find the other ip, skip the space */
     ips=dash+1;
-    while( *ips == ' ' ) {
+/*XXX the bug: tabs are also valid whitespace */    
+/*    while( *ips == ' ' ) { */
+    while(isspace((int)*ips)) {
       ips++;
     }
     
