@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.15 $
+  $Revision: 1.18 $
 
   Socket module - routines facilitating calls to socket library.
 
@@ -309,7 +309,10 @@ er_ret_t SK_connect(int  *sock, char *hostname, unsigned int port, unsigned int
     close(s);
     return SK_TIMEOUT;
   }
-  dieif( sel < 0 ); /* we don't expect problems with select */
+  if (sel < 0) {
+    close(s);
+    return SK_CONNECT;
+  }
   
   gs = getsockopt(s, SOL_SOCKET, SO_ERROR, &er, &erlen);
   
@@ -367,40 +370,99 @@ int SK_read(int sockfd, char *buf, size_t count) {
 /* SK_write() */
 /*++++++++++++++++++++++++++++++++++++++
 
-   This is just like the write() system call, except that it will
-   make sure that all data is transmitted.
+   int SK_write   Returns:
+                     -1 on error
+		      0 on timeout 
+		      1 on success (all bytes written)
 
-   int SK_write   Returns the number of bytes written.
+   int sockfd                      The socket file descriptor.
 
-   int    sockfd  The socket file descriptor.
+   const char *buf                 The buffer to be written to the socket.
 
-   char   *buf    The buffer to be written to the socket.
+   int count                       The number of bytes in the buffer.
 
-   size_t count   The number of bytes in the buffer.
+   const struct timeval *timeout   Maximum time to wait between each 
+                                   write() call, or NULL for "forever".
 
-  Author:
-        ottrey
+   int *count_sent                 Set to the number of bytes sucessfully
+                                   written.  This value is always valid, 
+				   although it will be less than the 
+				   total number of bytes to send if 
+				   the function returns -1 or 0.  NULL
+				   may be sent if the caller does not
+				   care about the number of bytes sent on
+				   failure.
 
   ++++++++++++++++++++++++++++++++++++++*/
-int SK_write(int  sockfd, const char *buf, size_t count) {
-  size_t  bytes_sent = 0;
-  int     this_write;
 
+int 
+SK_write(int sockfd, 
+         const char *buf,
+	 int count,
+	 const struct timeval *timeout, 
+         int *count_sent) 
+{
+  int     local_count_sent;    /* only used if caller passes NULL */
+  fd_set  fds;
+  int     select_ret;
+  struct  timeval tv;
+  int     write_ret;
   
+  /* copious logging never hurt anybody (well, except people with slow
+     computers or small hard disks) */
   ER_dbg_va(FAC_SK, ASP_SK_WRIT,
 	    "SK_write = { sockfd=[%d], buf=[%s], count=[%d]", 
 	    sockfd, buf, count);
 
-  while (bytes_sent < count) {
-    do
-      this_write = write(sockfd, buf, count - bytes_sent);
-    while ( (this_write < 0) && (errno == EINTR) );
-    if (this_write <= 0)
-      return this_write;
-    bytes_sent += this_write;
-    buf += this_write;
+  /* allow caller to pass NULL if it doesn't care about the count sent */
+  if (count_sent == NULL) {
+      count_sent = &local_count_sent;
   }
-  return count;
+
+  /* loop around until we send all of our data, or we get a timeout or
+     disconnect */
+  *count_sent = 0;
+  while (*count_sent < count) {
+
+    /* first, call select() and see if we can write without blocking */
+    FD_ZERO(&fds);
+    FD_SET(sockfd, &fds);
+    if (timeout == NULL) {
+        select_ret = select(sockfd+1, NULL, &fds, NULL, NULL);
+    } else {
+        /* make a copy of timeout, because it's value is undefined 
+	   on return from select() */
+        tv = *timeout;
+        select_ret = select(sockfd+1, NULL, &fds, NULL, &tv);
+    }
+    /* check for error */
+    if (select_ret < 0) {
+        /* log the error and return "timeout" */
+	ER_perror(FAC_SK, SK_SELECT, "(%d) %s", errno, strerror(errno));
+        return -1;
+    }
+    /* check for timeout */
+    if ((select_ret == 0) || !FD_ISSET(sockfd, &fds)) {
+        return -1;
+    }
+
+
+    /* at this point we can safely write */
+    write_ret = write(sockfd, buf, count - *count_sent);
+
+    /* if write failed, assume other side disconnected */
+    if (write_ret <= 0) {
+        return 0;
+    }
+
+    /* update our current status */
+    *count_sent += write_ret;
+    buf += write_ret;
+
+  }
+
+  /* all bytes sent, return success */
+  return 1;
 } /* SK_write() */
 
 
@@ -478,26 +540,25 @@ int SK_gets(int  sockfd, char *str, size_t count) {
    This function writes a character string out to a socket.
 
    int SK_puts  The total_count of bytes written, 
-                or errors (represented as negative numbers)
+                or -1 on hangup, error, or timeout
 
    int    sockfd    The socket file descriptor.
 
    char   *str      The buffer to be written from the socket.
 
-  More:
-  +html+ <PRE>
-  Authors:
-        ottrey
-
-  Side Effects:
-        This function will return -1 if the socket is closed during the write operation.
-  +html+ </PRE>
+   const struct timeval *timeout   Maximum time to wait between each 
+                                   write() call, or NULL for "forever".
 
   ++++++++++++++++++++++++++++++++++++++*/
-int SK_puts(int  sockfd, const char *str) {
-
-  return SK_write(sockfd, str, strlen(str));
-
+int 
+SK_puts(int sockfd, const char *str, const struct timeval *timeout) 
+{
+  int count_sent;
+  if (SK_write(sockfd, str, strlen(str), timeout, &count_sent) <= 0) {
+      return -1;
+  } else {
+      return count_sent;
+  }
 } /* SK_puts() */
 
 /* SK_putc() */
@@ -511,9 +572,16 @@ int SK_puts(int  sockfd, const char *str) {
 
    char ch           character
 
+   const struct timeval *timeout   Maximum time to wait between each 
+                                   write() call, or NULL for "forever".
+
   ++++++++++++++++++++++++++++++++++++++*/
-int SK_putc(int  sockfd, char ch) {
-  return SK_write(sockfd, &ch, 1);
+int 
+SK_putc(int sockfd, char ch, const struct timeval *timeout) 
+{
+  int count_sent;
+  SK_write(sockfd, &ch, 1, timeout, &count_sent);
+  return count_sent;
 }/* SK_putc() */
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -570,7 +638,13 @@ char *SK_getpeername(int  sockfd)
       hostaddress = wr_string("--");
     }
     else {
-      return NULL;
+      /* XXX: hack to avoid crash in the case where peer disconnects */
+      /* To fix this, the socket interface needs to deal with a structure
+         containing not only the file descriptor, but also the address of
+	 the host and the peer.  Other goodies, like the current time, 
+	 time of last operation, and last errno could also be included
+	 for good measure. */
+      hostaddress = UT_strdup("127.0.0.1");
     }
   }
 
@@ -607,6 +681,18 @@ int SK_getpeerip(int  sockfd, ip_addr_t *ip) {
   if (getpeername(sockfd, (struct sockaddr *)(& addr_in), &namelen) != -1) {
     ret=0;
     IP_addr_s2b(ip, &addr_in, namelen);
+  } else {
+      /* XXX: hack to avoid crash in the case where peer disconnects */
+      /* To fix this, the socket interface needs to deal with a structure
+         containing not only the file descriptor, but also the address of
+	 the host and the peer.  Other goodies, like the current time, 
+	 time of last operation, and last errno could also be included
+	 for good measure. */
+      ret = 0;
+      addr_in.sin_family = AF_INET;
+      addr_in.sin_addr.s_addr = INADDR_LOOPBACK;
+      addr_in.sin_port = 0;
+      IP_addr_s2b(ip, &addr_in, namelen);
   }
   
   return ret;
