@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.36 $
+  $Revision: 1.37 $
 
   SQL module (sq) - this is a MySQL implementation of the SQL module.
 
@@ -1068,3 +1068,63 @@ int SQ_ping(SQ_connection_t *sql_connection)
 	return(mysql_ping(sql_connection));
 }
 
+/************************************************************
+* get_minmax_id()                                           *
+*                                                           *
+* Returns the min or max ID of the table                    *
+*                                                           *
+* Returns:                                                  *
+*  min (max=0) or max (max=1) ID                            *
+*  -1 in case of an error                                   *
+*                                                           *
+*                                                           *
+*************************************************************/
+
+long sq_get_minmax_id(SQ_connection_t *sql_connection, char *id_name, char *table, int max)
+{
+char query[STR_M];
+SQ_result_set_t *sql_result;
+SQ_row_t *sql_row;
+char *sql_str;
+long id;
+char *minmax;
+int sql_err;
+
+if(max==1)minmax="max"; else minmax="min";
+
+ sprintf(query, "SELECT %s(%s) FROM %s ", minmax, id_name, table);
+
+ sql_err = SQ_execute_query(sql_connection, query, &sql_result);
+ 
+ if(sql_err) {
+    ER_perror(FAC_SQ, SQ_CNCT, "%s[%s]\n", SQ_error(sql_connection), query);
+    die;
+ }
+        
+	 
+ if ((sql_row = SQ_row_next(sql_result)) != NULL) {
+	sql_str = SQ_get_column_string(sql_result, sql_row, 0);
+
+     /* We must process all the rows of the result,*/
+     /* otherwise we'll have them as part of the next qry */
+	while ( (sql_row = SQ_row_next(sql_result)) != NULL) {
+	  ER_perror(FAC_SQ, SQ_CNCT, "duplicate max [%s]\n", query);
+	  die;
+	}
+ }
+ else sql_str=NULL;
+ 
+ if(sql_result){ SQ_free_result(sql_result); sql_result=NULL; }
+ 
+ if(sql_str) {
+  id = atol(sql_str);
+  free(sql_str);
+ }
+ /* table is empty, max_id=min_id=0 */
+ else id=0;
+ 
+ return(id);
+ 
+}
+
+
