@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.25 $
+  $Revision: 1.26 $
 
   rollback(), commit(), delete() - rollback, commit update transaction, delete an object
 
@@ -341,21 +341,21 @@ if(ACT_DELETE(tr->action)) return(0);
    g_string_sprintf(query, "LOCK TABLES ");
    
    /* we need to lock tables for mntner and role differently, otherwise there will be duplicates (names names, etc.)*/
-   if((tr->class_type!=C_MT) && (tr->class_type!=C_RO)){
-    g_string_sprintfa(query, " %s WRITE,",  DF_get_class_sql_table(tr->class_type));
-    
-    for (i=0; tables[tr->class_type][i] != NULL; i++) 
-      g_string_sprintfa(query, " %s WRITE,", tables[tr->class_type][i]);
-   } else { /* mntner and role are special cases */
-      g_string_sprintfa(query, " mntner WRITE, person_role WRITE, ");
-   }
+/*   if((tr->class_type!=C_MT) && (tr->class_type!=C_RO)){ */
+   g_string_sprintfa(query, " %s WRITE,",  DF_get_class_sql_table(tr->class_type));
    
-    for (i=TAB_START; tables[tr->class_type][i] != NULL; i++)
+   if((tr->class_type==C_RO)) g_string_sprintfa(query, " mntner WRITE, ");
+   else if((tr->class_type==C_MT)) g_string_sprintfa(query, " person_role WRITE, names WRITE, "); 
+    else
+     for (i=0; tables[tr->class_type][i] != NULL; i++) 
+        g_string_sprintfa(query, " %s WRITE,", tables[tr->class_type][i]);
+   
+   for (i=TAB_START; tables[tr->class_type][i] != NULL; i++)
       g_string_sprintfa(query, " %s WRITE,", tables[tr->class_type][i]);
     
-    g_string_sprintfa(query, " last WRITE, history WRITE, transaction_rec WRITE ");
+   g_string_sprintfa(query, " last WRITE, history WRITE, transaction_rec WRITE ");
     
-    sql_err = SQ_execute_query(tr->sql_connection, query->str, (SQ_result_set_t **)NULL);
+   sql_err = SQ_execute_query(tr->sql_connection, query->str, (SQ_result_set_t **)NULL);
 
 
   /* Perform first phase - deletions */
