@@ -1,6 +1,6 @@
 /***************************************
   
-  $Revision: 1.38 $
+  $Revision: 1.40 $
 
   Core functions for update lower layer 
 
@@ -70,11 +70,16 @@ static int auth_member_of(Attribute_t *attr, Transaction_t *tr);
 /* Convert ifaddr attribute into numbers */
 er_ret_t convert_if(char *avalue, unsigned int *pif_address)
 {
-char *delim;
+char *delim=avalue;
 ip_addr_t ip_addr;
 er_ret_t ret;
 
-  if ((delim=index(avalue, ' '))!=NULL) *delim='\0';
+/*XXX  if ((delim=index(avalue, ' '))!=NULL) *delim='\0'; */
+  while((*delim) && (!isspace((int)*delim))) {
+      delim++;
+  }
+  *delim='\0';  
+  
   ret=IP_addr_a2v4(avalue, &ip_addr,  pif_address );
   return(ret);
 }
@@ -90,8 +95,12 @@ char *host;
   host=NULL;
   strcpy(buff, avalue);
   g_strchug(buff);
-  delim=index(buff, ' ');
-  *delim='\0';
+/*XXX  delim=index(buff, ' '); */
+  delim=buff;
+  while((*delim) && (!isspace((int)*delim))) {
+      delim++;
+  }
+  if (*delim)*delim='\0'; else return(NULL);
   delim++; 
 
 /* convert the type      */
@@ -102,8 +111,13 @@ char *host;
 
   token=delim;
   g_strchug(token);
-  delim=index(token, ' ');
-  if(delim){
+/*XXX  delim=index(token, ' '); */
+  delim=token;
+  while((*delim) && (!isspace((int)*delim))) {
+      delim++;
+  }
+  
+  if(*delim){
    *delim='\0';
    delim++; 
   }	      
@@ -111,7 +125,7 @@ char *host;
   host = g_strdup(token);
       
 /* convert port number      */
-  if(delim){
+  if(*delim){
     token=delim;	
     *port = atoi(token);
     if (*port==0) *port=RF_DEF_PORT; /* default port number*/
@@ -124,7 +138,8 @@ char *host;
 static int convert_as(char *as)
 {
 char *ptr;
- ptr=as; ptr++; ptr++; 
+ ptr=as; 
+ while((*ptr) && (!isdigit((int)*ptr))) ptr++;
  return(atoi(ptr));   
 }
 
@@ -136,8 +151,8 @@ char *token;
   
   range=g_strdup(as_range);
   token=range;
-  *begin=convert_as(strsep(&token, " -"));
-  *end=convert_as(strsep(&token, " -"));
+  *begin=convert_as(strsep(&token, "-"));
+  *end=convert_as(token);
   free(range);
   return(0);
 }
@@ -476,8 +491,16 @@ int object_type=-1;
 /* thus, the check is simple - if there is a space - not a nh */
 static int isnichandle(char *name)
 {
- if(index(name, ' ')) return(0);
- else return(1);	
+ char *delim;
+ 
+/*XXX if(index(name, ' ')) return(0);
+ else return(1);	*/
+ delim=name;
+ while((*delim) && (!isspace((int)*delim))) {
+      delim++;
+ }
+ if(*delim) return(0);
+ else return(1);
 }
 
 
@@ -695,7 +718,7 @@ char *token=NULL;
 
   /* Insert dummy in the last table */
   /* Calculate the object_id - should be max+1 */
-  dummy_id = get_minmax_id(tr->sql_connection, "object_id", "last", 1) +1;
+  dummy_id = SQ_get_max_id(tr->sql_connection, "object_id", "last") +1;
  /* Record dummy's object_id, it'll be needed in commit/rollback */
   tr->dummy_id[tr->ndummy]=dummy_id; tr->ndummy++;
 
@@ -706,8 +729,8 @@ char *token=NULL;
 
   sprintf(str_id, "%ld", tr->object_id);
   timestamp=time(NULL);
-  sprintf(query, "INSERT INTO last SET thread_id=%d, timestamp=%ld, object_type=%d, object='DUMMY for %s'", 
-                  tr->thread_ins, timestamp, DUMMY_TYPE, str_id);
+  sprintf(query, "INSERT INTO last SET thread_id=%d, object_id=%ld, timestamp=%ld, object_type=%d, object='DUMMY for %s'", 
+                  tr->thread_ins, dummy_id, timestamp, DUMMY_TYPE, str_id);
   
   ER_dbg_va(FAC_UD, ASP_UD_SQL, "%s [%s]", UD_TAG, query);
   sql_err = SQ_execute_query(tr->sql_connection, query, (SQ_result_set_t **)NULL);
@@ -718,9 +741,6 @@ char *token=NULL;
    die;
   }	
 	
-  /* check that the dummy_id is correct */
-  if(dummy_id != SQ_get_insert_id(tr->sql_connection)) die; /* probably implementation of autoincrement changed */
-
    
   /* compose the query */
   query_type=DF_get_dummy_query_type(attr->type);
@@ -762,7 +782,8 @@ char *token=NULL;
     query_fmt = DF_get_insert_query(A_PN);
     attr_value = g_strdup(attr->value); 
     token = attr_value;
-    while((p_name=strsep(&token, " "))){
+    while((p_name=strsep(&token, " \t"))){
+	if (*p_name != '\0'){
 		sprintf(query, query_fmt, tr->thread_ins, dummy_id, DUMMY_TYPE, p_name);
 		ER_dbg_va(FAC_UD, ASP_UD_SQL, "%s [%s]", UD_TAG, query);
 		sql_err = SQ_execute_query(tr->sql_connection, query, (SQ_result_set_t **)NULL);
@@ -771,6 +792,7 @@ char *token=NULL;
 		  ER_perror(FAC_UD, UD_SQL, "insert dummy names:%s[%s]\n", SQ_error(tr->sql_connection), query);
 		  result=-1;
 		 }
+	}
     }
     free(attr_value);
   }
@@ -866,7 +888,7 @@ char *mu_mntner;
          case UD_AX_MU: /* for mnt_routes table*/
    	                a_value=g_strdup(attr->value); 
                         token = a_value;
-                        mu_mntner=strsep(&token, " ");
+                        mu_mntner=strsep(&token, " \t");
                        	if (!IS_DUMMY_ALLOWED(tr->mode))condition="AND dummy=0 "; else condition=NULL;
 	 		sprintf(query, query_fmt, tr->thread_upd, tr->object_id, 
 	 			get_ref_id(tr, "mntner", "mntner", mu_mntner, condition));
@@ -899,8 +921,17 @@ char *mu_mntner;
     			break;
 	 case UD_LF_RF:
 			rf_host=convert_rf(attr->value, &rf_type, &rf_port);
-			sprintf(query, query_fmt, tr->thread_upd, tr->object_id, rf_type, rf_host, rf_port);
-			if(rf_host)free(rf_host);
+			if(rf_host == NULL) {
+				tr->error|=ERROR_U_OBJ;
+  	                        tr->succeeded=0;
+  	                        g_string_sprintfa(tr->error_script,"E[%d][%d:%s]:incorrect attribute value\n" ,
+				ERROR_U_OBJ, attr->type, attr->value);
+				return;
+			}
+			else {
+			        sprintf(query, query_fmt, tr->thread_upd, tr->object_id, rf_type, rf_host, rf_port);
+			        free(rf_host);
+			}
 			break;			
   	 case UD_LF_AY:
                   	sprintf(query, query_fmt, tr->thread_upd, tr->object_id, convert_time(attr->value));
@@ -987,7 +1018,7 @@ char * rf_host; /* needs to be freed after use*/
 int rf_type, rf_port;
 char *a_value;
 int sq_info[3];
-char *mu_mntner, *mu_prefix;
+char *mu_mntner;
 int dummy_err;
 char *sq_error;
 ip_prefix_t dn_pref;
@@ -1125,8 +1156,7 @@ char *token;
    case UD_AX_MU: /* for mnt_routes table*/
    		a_value=g_strdup(attr->value); 
 		token = a_value;
-		mu_mntner=strsep(&token, " ");
-		mu_prefix=token;
+		mu_mntner=strsep(&token, " \t");
    		sprintf(query, query_fmt, tr->thread_ins, tr->object_id, tr->class_type, mu_mntner);
    		free(a_value);
    		if (!IS_DUMMY_ALLOWED(tr->mode))strcat(query, " AND dummy=0 ");
@@ -1158,8 +1188,17 @@ char *token;
     		break;
    case UD_LF_RF: /* for refer table*/
     		rf_host=convert_rf(attr->value, &rf_type, &rf_port);
-    		sprintf(query, query_fmt, tr->thread_ins, tr->object_id, rf_type, rf_host, rf_port);
-    		if(rf_host)free(rf_host);
+		if(rf_host == NULL) {
+				tr->error|=ERROR_U_OBJ;
+  	                        tr->succeeded=0;
+  	                        g_string_sprintfa(tr->error_script,"E[%d][%d:%s]:incorrect attribute value\n" ,
+				ERROR_U_OBJ, attr->type, attr->value);
+				return;
+		}
+		else {
+    		                sprintf(query, query_fmt, tr->thread_ins, tr->object_id, rf_type, rf_host, rf_port);
+    		                free(rf_host);
+		}
     		break;	
    case UD_LF_AY: /* for auth_override table*/
    		sprintf(query, query_fmt, tr->thread_ins, tr->object_id, convert_time(attr->value));
@@ -1384,7 +1423,7 @@ static int perform_create(Transaction_t *tr)
  GString *query;
  long timestamp;
  int sql_err;
- long object_id;
+// long object_id;
   
   
  if ((query = g_string_sized_new(STR_XL)) == NULL){
@@ -1400,14 +1439,17 @@ static int perform_create(Transaction_t *tr)
       str = (obj->object)->str;
       timestamp=time(NULL);
       tr->sequence_id=1; /* we start with 1*/
-      /* Calculate the object_id - should be max+1 */
-      tr->object_id = get_minmax_id(tr->sql_connection, "object_id", "last", 1);
-      /* if last is empty, start with 1, otherwise the assign the next id */
-      if(tr->object_id==-1)tr->object_id=1; else tr->object_id++;
+  /* Calculate the object_id - should be max+1 */
+  /* XXX we cannot use autoincrement with MyISAM tables */
+  /* XXX because they keep the max inserted id even if  */
+  /* XXX it was deleted later, thus causing gaps we don't want */
+ 
+      tr->object_id = SQ_get_max_id(tr->sql_connection, "object_id", "last") +1;
       TR_update_id(tr);
 
-      g_string_sprintf(query, "INSERT INTO last SET thread_id=%d, timestamp=%ld, sequence_id=1, object_type=%d, object='%s', pkey='%s' ",
-      	              tr->thread_ins, timestamp, tr->class_type, str, tr->K->str);
+      g_string_sprintf(query, "INSERT INTO last SET thread_id=%d, object_id=%ld, " 
+                              "timestamp=%ld, sequence_id=1, object_type=%d, object='%s', pkey='%s' ",
+      	              tr->thread_ins, tr->object_id, timestamp, tr->class_type, str, tr->K->str);
 
       ER_dbg_va(FAC_UD, ASP_UD_SQL, "%s [%s]", UD_TAG, query->str);
       sql_err = SQ_execute_query(tr->sql_connection, query->str, (SQ_result_set_t **)NULL);
@@ -1420,10 +1462,6 @@ static int perform_create(Transaction_t *tr)
 	die;
       }
       else {
-      /* Get generated (autoincrement) object_id */
-        object_id=SQ_get_insert_id(tr->sql_connection);
-	/* compare it with the calculated one */
-	if(tr->object_id != object_id) die; /* probably implementation of autoincrement changed */
         g_slist_foreach(obj->attributes, each_attribute_process, tr);
       }
     g_string_free(query, TRUE);
