@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.29 $
+  $Revision: 1.34 $
 
   Wrapper for NRTM client
 
@@ -51,6 +51,9 @@
 #include "protocol_mirror.h"
 #include "ta.h"
 
+#include "iproutines.h"
+#include "aa.h"
+
 /* here we store sockets for update threads */
 /* they are from SV module */
 extern int SV_update_sock[];
@@ -64,7 +67,7 @@ extern int SV_update_sock[];
 /* Maximum number of objects(serials) we can consume at a time */
 #define SBUNCH 1000
 
-/* Timeout in seconds when reading from DBupdate */
+/* Timeout in seconds when reading (writing) from DBupdate */
 #define STREAM_TIMEOUT 120
 
 /************************************************************
@@ -93,7 +96,7 @@ struct sockaddr_in serv_addr;
 struct in_addr *paddr;
 char line_buff[STR_XXL];
 int fd;
-int nwrite;
+int ret;
 struct hostent result;
 int error;
 int network;
@@ -119,7 +122,7 @@ int network;
    memcpy(&serv_addr.sin_addr, paddr, sizeof(struct in_addr));
 /*   fprintf(stderr,"Trying %s port %d\n", inet_ntoa(serv_addr.sin_addr), nrtm->port);*/
    if(connect(sockfd, (struct sockaddr *)&serv_addr, sizeof(serv_addr))==-1) {
-     ER_perror(FAC_UD, UD_FS, "cannot cannect"); 
+     ER_perror(FAC_UD, UD_FS, "cannot connect"); 
      close(sockfd);
      return(-1);
    }  
@@ -132,8 +135,8 @@ int network;
       sprintf(line_buff, "-k -g %s:%d:%ld-LAST\n", source, nrtm->version, nrtm->current_serial+1);
    else
       sprintf(line_buff, "-g %s:%d:%ld-%ld\n", source, nrtm->version, nrtm->current_serial+1, nrtm->current_serial+SBUNCH);   
-   nwrite=SK_write(sockfd, line_buff, strlen(line_buff) );
-   if(nwrite != strlen(line_buff)) { 
+   ret=SK_write(sockfd, line_buff, strlen(line_buff), NULL, NULL );
+   if(ret != 1) { 
 	   ER_perror(FAC_UD, UD_FS, "cannot write");
 	   close(sockfd);
 	   return(-1); 
@@ -299,7 +302,8 @@ char *source_name = ca_get_srcname(source_hdl);
      die;
    }
 
-   ER_inf_va(FAC_UD, ASP_UD_UPDLOG, "%s connecting to NRTM server (current serial=%ld)", UD_TAG, nrtm->current_serial);
+   ER_inf_va(FAC_UD, ASP_UD_UPDLOG, "%s connecting to NRTM server [%s:%d] (current serial=%ld)", 
+                                    UD_TAG, nrtm->server, ntohs(nrtm->port), nrtm->current_serial);
 
   /* Get file descriptor of the data stream (RPSL format, use mirror reflector to convert if needed)*/
     nrtm_fd=get_NRTM_fd(nrtm, upto_last, source_name);
@@ -348,9 +352,11 @@ char *source_name = ca_get_srcname(source_hdl);
      if(num_ok==0) upto_last=1;
      /* get delay */
      nrtm_delay=ca_get_srcnrtmdelay(source_hdl);
+   } else 
+     /* we need to delay the next attempt not to have a birst of requests */
+     nrtm_delay=TIMEOUT; 
      /* sleep the delay seconds or untill the shutdown requested */
      SV_sleep(nrtm_delay);
-   } /* otherwise - no delay at all */
    
   } /* if do_updates */
   else SV_sleep(TIMEOUT); 
@@ -396,19 +402,22 @@ int listening_socket = SV_update_sock[source];
 int connected_socket;
 UD_stream_t ud_stream;
 int do_update=1;
-int do_server;
 int num_ok;
 ca_dbSource_t *source_hdl = ca_get_SourceHandleByPosition(source);
 char *db_host, *db_name, *db_user, *db_passwd;
 int db_port;
+ip_addr_t address;
+char *source_name;
+
+   source_name = ca_get_srcname(source_hdl);  
 
   { /* set up the lohgging path */
    /* get source we are going to update */
-   char *source_name = ca_get_srcname(source_hdl);  
    int res;
    char *er_ud_def = ca_get_er_ud_def; /* something like 'RIPUPDLOG basename' */
    char er_def[256];
    char *erret = NULL;
+   
 
    sprintf(er_def, "%s %s", er_ud_def, source_name);
    if( (res = ER_macro_spec(er_def, &erret)) != 0 ) {
@@ -418,7 +427,6 @@ int db_port;
    }     
    free(erret); /* the response is allocated and must be freed */
    free(er_ud_def);
-   free(source_name);
   } 
 
 /* get mode of operation: protected/unprotected (dummy) */
@@ -435,7 +443,7 @@ int db_port;
    else fprintf(stderr, "* running as a server\n");
 
 
-/* get error log facility */
+/* get the database */
   db_host = ca_get_srcdbmachine(source_hdl);
   db_port = ca_get_srcdbport(source_hdl);
   db_name = ca_get_srcdbname(source_hdl);
@@ -461,16 +469,39 @@ int db_port;
 
 do { /* be alive while do_server is 1. do_server is turned off by SIGINT */
  
-  /* make a record for thread accounting */
-  TA_add(listening_socket, "update");
-  TA_setactivity("waiting");
-  
+   /* make a record for thread accounting */
+   TA_add(listening_socket, "update");
+   TA_setactivity("waiting");
  
-/* accept connection */
+   /* accept connection */
    connected_socket = SK_accept_connection(listening_socket);
-   if(connected_socket==-1) break;
+   if(connected_socket==-1) continue;
    
-
+   /* check if the client is authorised to update */
+   SK_getpeerip(connected_socket, &address);
+   if(!AA_can_ripupdate(&address, source_name)){
+   char *hostaddress;
+   sk_conn_st condat;
+   char buff[STR_L];
+    
+    memset( &condat, 0, sizeof(sk_conn_st));
+    condat.wr_timeout.tv_sec=STREAM_TIMEOUT;
+    condat.sock = connected_socket;
+    SK_getpeerip(connected_socket, &(condat.rIP));
+    memcpy( &(condat.eIP), &(condat.rIP), sizeof(ip_addr_t));
+    hostaddress = SK_getpeername(connected_socket);
+    condat.ip = hostaddress;
+    
+    ER_inf_va(FAC_UD, ASP_UD_UPDLOG, "[%s] --  Not authorized to update the source %s", hostaddress, source_name);
+    sprintf(buff, "\n%%ERROR:406: not authorized to update the database\n\n\n");
+    SK_cd_puts(&condat, buff);
+    SK_cd_close(&(condat));
+    
+    free(hostaddress);
+    /* start the next loop */
+    continue;
+   }
+    
    /* make a record for thread accounting */
    TA_delete(); /* Delete 'waiting' record */
    TA_add(connected_socket, "update");
@@ -479,47 +510,45 @@ do { /* be alive while do_server is 1. do_server is turned off by SIGINT */
    ud_stream.condat.sock = connected_socket;
    ud_stream.condat.rtc = 0;
 
- do_update=CO_get_do_update();
- if(do_update) {
- 
-   TA_setactivity("suspended");
    
    ER_dbg_va(FAC_UD, ASP_UD_UPDLOG, "%s Connection accepted...", UD_TAG);
   
-  ud_stream.log.num_ok=0; 
-  ud_stream.log.num_failed=0;
+   ud_stream.log.num_ok=0; 
+   ud_stream.log.num_failed=0;
  
-  /* Check connection to the database and try to reconnect*/
- if(SQ_ping(ud_stream.db_connection)) {
-   ER_perror(FAC_UD, UD_SQL, "%s", SQ_error(ud_stream.db_connection));
-   die;
-  }
-
-  ER_dbg_va(FAC_UD, ASP_UD_UPDLOG, "%s starting processing object", UD_TAG);
   
-/***************** process stream ****************/
+   while((do_update=CO_get_do_update())!=1) {
+	   TA_setactivity("suspended");
+	   /* if shutdown was requested - break */
+	   if(SV_sleep(3*TIME_SLICE)) break;
+   }
 
-    num_ok=UD_process_stream(&ud_stream);
+   if(do_update) {
+     /* Check connection to the database and try to reconnect*/
+     if(SQ_ping(ud_stream.db_connection)) {
+       ER_perror(FAC_UD, UD_SQL, "%s", SQ_error(ud_stream.db_connection));
+       die;
+     }
 
-/***************** process stream ****************/    
+     ER_dbg_va(FAC_UD, ASP_UD_UPDLOG, "%s starting processing object", UD_TAG);
   
-  ER_dbg_va(FAC_UD, ASP_UD_OBJ, "%s processing object finished", UD_TAG);
+     /***************** process stream ****************/
+
+      num_ok=UD_process_stream(&ud_stream);
+
+     /***************** process stream ****************/    
   
-  /* close the socket of the NRTM stream */
-   close(ud_stream.condat.sock);
+      ER_dbg_va(FAC_UD, ASP_UD_OBJ, "%s processing object finished", UD_TAG);
+  
+     /* close the socket of the NRTM stream */
+     close(ud_stream.condat.sock);
     
- }  /* if do_update*/
-else { /* Otherwise print a message*/
- /* To display with 'show threads' */
-  TA_setactivity("suspended");
- 
- }
-  /* make a record for thread accounting */
+   }  /* if do_update*/
+
+   
    TA_delete();
 
-   do_server=CO_get_do_server();  
-
-} while (do_server);  /* main cycle */
+} while (CO_get_do_server());  /* main cycle */
   
 /*   fclose(ud_stream.log.logfile); */
  /* That's all. Close connection to the DB */ 
@@ -528,6 +557,7 @@ else { /* Otherwise print a message*/
  free(db_name);
  free(db_user);
  free(db_passwd);
+ free(source_name);
 
 
  ER_inf_va(FAC_UD, ASP_UD_UPDLOG, "%s update server stopped", UD_TAG);
