@@ -18,6 +18,12 @@ char * UP_remove_EOLs(char * arg);
 
 char * UP_replace_strings(char * str, const char * erase_str, const char * insert_str);
 
+char * UP_put_assigned_NIC(char * arg, const char * assigned_NIC);
+
+char * UP_get_current_date();
+
+char * filter_out_same_origins(char * objects, char * object);
+
 #ifdef __cplusplus
 }
 #endif
@@ -29,3 +35,5 @@ char * UP_replace_globals(const char * arg);
 GSList * get_updto_vector(GSList * mntners);
 
 char * delete_delete_attrib(char * arg);
+
+char * delete_override(char * arg);
