@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.33 $
+  $Revision: 1.39 $
 
   History/Serial Cleanup (hs_cleanup). This utility archives serials
   and history entries, and deletes them from the live database.
@@ -8,7 +8,7 @@
 
   ******************/ /******************
   Filename            : hs_cleanup.c
-  Authors             : Daniele Arena
+  Authors             : Daniele Arena, and the RIPE DBM staff
   OSs Tested          : Solaris 7
   ******************/ /******************
   Copyright (c) 2000, 2001                        RIPE NCC
@@ -158,6 +158,7 @@ long crashing_serial;
 int crash_position;
 int code_location;
 int case_branch;
+int Test = 0;
 
 
 /********** FUNCTION DEFINITIONS **********/
@@ -280,9 +281,12 @@ int main (int argc, char *argv[])
 
   /* Get options */
 
-  while ((ch = getopt(argc, argv, "?T:S:M:H:D:W:h:P:u:p:d:")) != EOF )
+  while ((ch = getopt(argc, argv, "?tT:S:M:H:D:W:h:P:u:p:d:")) != EOF )
     switch((char)ch)
       {
+      case 't':
+        Test = 1;
+	break;
       case 'T':
 	if (tflg)
 	  errflg++;
@@ -466,10 +470,11 @@ void usage(char *argv[])
 {
 
   printf ("Usage: \n\n");
-  printf ("  %s [-?] [-h host] [-P port] [-u user] [-p password] [-d database]\n", argv[0]);
+  printf ("  %s [-t] [-?] [-h host] [-P port] [-u user] [-p password] [-d database]\n", argv[0]);
   printf ("     [-T date|-S seconds|-M minutes|-H hours|-D days|-W weeks] \n");
 
   printf ("\nGeneral options:\n");
+  printf ("   -t: Run in test mode (no database updates)\n");
   printf ("   -?: This text\n");
 
   printf ("\nSQL Options:\n");
@@ -510,66 +515,77 @@ int get_highest_serial_before_date (long *highest_serial_ptr, long date)
   char query[MAXCMDLEN];
   SQ_result_set_t *result;
   SQ_row_t *row;
-  long highest_serial_last, highest_serial_history;
+  long highest_serial_last;
+  long highest_serial_history;
+  long highest_serial_history_add, highest_serial_history_del;
 
-  /* sprintf (query, "SELECT MAX(serials.serial_id) FROM history,serials 
-			WHERE history.timestamp < %ld 
-			AND history.object_id = serials.object_id 
-			AND history.sequence_id = serials.sequence_id ", date); */
-
-  
-/*    sprintf (query, "SELECT MAX(serials.serial_id) 
-			FROM serials NATURAL LEFT JOIN last NATURAL LEFT JOIN history
-			WHERE ((last.timestamp < %ld 
-				AND last.object_id = serials.object_id 
-				AND last.sequence_id = last.sequence_id)
-			OR (history.timestamp < %ld 
-				AND history.object_id = serials.object_id 
-				AND history.sequence_id = serials.sequence_id))",
-	   date, date);
-
-printf(query);
-
-  execute_sql_query(query, &result);
-  if ( (row = SQ_row_next(result)) != NULL )
-    {
-      *highest_serial_ptr = row[0] ? atol((const char *)row[0]) : 0;
-    }
-
-  SQ_free_result(result);
-
-*/
-    sprintf (query, "SELECT MAX(serials.serial_id) 
-                     FROM serials, last
-                     WHERE (last.object_id = serials.object_id 
-		     AND last.sequence_id = serials.sequence_id 
-		     AND last.timestamp < %ld)", date);
+    sprintf (query, "SELECT MAX(serials.serial_id) "
+                    "FROM serials, last "
+                    "WHERE (last.object_id = serials.object_id "
+		    "AND last.sequence_id = serials.sequence_id "
+		    "AND last.timestamp < %ld) ", date);
   
   execute_sql_query(query, &result);
   if ( (row = SQ_row_next(result)) != NULL )
     {
       highest_serial_last = row[0] ? atol((const char *)row[0]) : 0;
     }
+printf("%s:%d highest_serial_last:%ld\n", __FILE__, __LINE__, highest_serial_last);
 
   SQ_free_result(result);
 
-    sprintf (query, "SELECT MAX(serials.serial_id) 
-                     FROM serials, history
-                     WHERE (history.object_id = serials.object_id 
-		     AND history.sequence_id = serials.sequence_id 
-		     AND history.timestamp < %ld)", date);
+    sprintf (query, "SELECT MAX(serials.serial_id) "
+                    "FROM serials, history "
+                    "WHERE (history.object_id = serials.object_id "
+		    "AND serials.operation = %d "
+		    "AND history.sequence_id = serials.sequence_id "
+		    "AND history.timestamp < %ld) ", OP_ADD, date);
   
   execute_sql_query(query, &result);
   if ( (row = SQ_row_next(result)) != NULL )
     {
-      highest_serial_history = row[0] ? atol((const char *)row[0]) : 0;
+      highest_serial_history_add = row[0] ? atol((const char *)row[0]) : 0;
     }
 
+printf("%s:%d highest_serial_history_add:%ld\n", __FILE__, __LINE__, highest_serial_history_add);
   SQ_free_result(result);
+
+    /************
+      For deleted objects, the sequence_id in the serial table is of the
+      deleted object, which contains the timestamp when it was CREATED,
+      not for the DELETION, which is what we care about.  So we look at
+      the object with the next sequence_id in the history table, which
+      was created at the same time this object was deleted.
+        - Shane
+     ************/
+
+    sprintf (query, "SELECT MAX(serials.serial_id) "
+                    "FROM serials, history "
+                    "WHERE (history.object_id = serials.object_id "
+		    "AND serials.operation = %d "
+		    "AND history.sequence_id = (serials.sequence_id+1) "
+		    "AND history.timestamp < %ld) ", OP_DELETE, date);
   
-  if(highest_serial_history>highest_serial_last)
-	  *highest_serial_ptr=highest_serial_history; 
-  else *highest_serial_ptr=highest_serial_last;
+  execute_sql_query(query, &result);
+  if ( (row = SQ_row_next(result)) != NULL )
+    {
+      highest_serial_history_del = row[0] ? atol((const char *)row[0]) : 0;
+    }
+printf("%s:%d highest_serial_history_del:%ld\n", __FILE__, __LINE__, highest_serial_history_del);
+
+  SQ_free_result(result);
+
+  if (highest_serial_history_add > highest_serial_history_del) {
+      highest_serial_history = highest_serial_history_add;
+  } else {
+      highest_serial_history = highest_serial_history_del;
+  }
+  
+  if(highest_serial_history>highest_serial_last) {
+      *highest_serial_ptr=highest_serial_history; 
+  } else { 
+      *highest_serial_ptr=highest_serial_last;
+  }
 
   return(0);
 
@@ -635,12 +651,12 @@ int archive_serials_and_history (long highest_serial)
   /* One word about the "<": Don't use "<=" because if highest_serial
      is the CURRENTSERIAL, it causes big problems to UD! 
      (at least in mirror mode...) */
-  sprintf (query, "SELECT serials.serial_id, serials.atlast, 
-			ELT(serials.atlast+1,'history','last','failed_transaction'), 
-			serials.object_id, serials.sequence_id, serials.operation 
-			FROM serials 
-			WHERE serials.serial_id < %ld
-			ORDER BY serials.serial_id", highest_serial);
+  sprintf (query, "SELECT serials.serial_id, serials.atlast, "
+		  "ELT(serials.atlast+1,'history','last','failed_transaction'),"
+		  "serials.object_id, serials.sequence_id, serials.operation "
+		  "FROM serials "
+		  "WHERE serials.serial_id < %ld "
+		  "ORDER BY serials.serial_id ", highest_serial);
   execute_sql_query(query, &result);
 
   /* Loop on every serial */
@@ -947,7 +963,8 @@ long fetch_timestamp_from_last (long obj_id, long seq_id)
   SQ_result_set_t *result;
   SQ_row_t *row;
 
-  sprintf (query, "SELECT timestamp FROM last WHERE object_id = %ld AND sequence_id = %ld", 
+  sprintf (query, "SELECT timestamp "
+                  "FROM last WHERE object_id = %ld AND sequence_id = %ld ", 
 	   obj_id, seq_id);
 
   execute_sql_query(query, &result);
@@ -977,7 +994,7 @@ long get_highest_object_id()
   SQ_result_set_t *result;
   SQ_row_t *row;
 
-  sprintf (query, "SELECT max(object_id) FROM last"); 
+  sprintf (query, "SELECT max(object_id) FROM last "); 
 
   execute_sql_query(query, &result);
   if ( (row = SQ_row_next(result)) != NULL)
@@ -1008,8 +1025,10 @@ int check_if_next_is_deletion (long obj_id, long seq_id)
   SQ_row_t *row;
   long serial = 0;
 
-  sprintf (query, "SELECT serial_id, atlast FROM serials 
-			WHERE object_id = %ld AND sequence_id = %ld AND operation = %d", 
+  sprintf(query, "SELECT serial_id, atlast FROM serials "
+		 "WHERE object_id = %ld AND "
+		 "      sequence_id = %ld AND "
+		 "      operation = %d ", 
 	   obj_id, seq_id, OP_DELETE);
 
   execute_sql_query(query, &result);
@@ -1042,29 +1061,29 @@ int create_archive_tables()
   char cmd[MAXCMDLEN];
 
   sprintf (cmd, 
-	   "CREATE TABLE history_archive (
-		object_id int(10) unsigned DEFAULT '0' NOT NULL,
-		sequence_id int(10) unsigned DEFAULT '0' NOT NULL,
-		serial int(11) DEFAULT '0' NOT NULL,
-		prev_serial int(11) DEFAULT '0' NOT NULL,
-		timestamp int(10) unsigned DEFAULT '0' NOT NULL,
-		object_type tinyint(3) unsigned DEFAULT '0' NOT NULL,
-		object longblob NOT NULL,
-		pkey varchar(64) default '' NOT NULL,
-		PRIMARY KEY (object_id,sequence_id,serial),
-		INDEX (serial)
-		);");
+	   "CREATE TABLE history_archive ( "
+		"object_id int(10) unsigned DEFAULT '0' NOT NULL, "
+		"sequence_id int(10) unsigned DEFAULT '0' NOT NULL, "
+		"serial int(11) DEFAULT '0' NOT NULL, "
+		"prev_serial int(11) DEFAULT '0' NOT NULL, "
+		"timestamp int(10) unsigned DEFAULT '0' NOT NULL, "
+		"object_type tinyint(3) unsigned DEFAULT '0' NOT NULL, "
+		"object longblob NOT NULL, "
+		"pkey varchar(64) default '' NOT NULL, "
+		"PRIMARY KEY (object_id,sequence_id,serial), "
+		"INDEX (serial) "
+		") ");
   create_table(cmd);
 
 
   sprintf (cmd, 
-	   "CREATE TABLE serials_archive (
-		serial_id int(11) DEFAULT '0' NOT NULL,
-		object_id int(10) unsigned DEFAULT '0' NOT NULL,
-		sequence_id int(10) unsigned DEFAULT '0' NOT NULL,
-		operation tinyint(4) unsigned DEFAULT '0' NOT NULL,
-		PRIMARY KEY (serial_id)
-		);");
+	   "CREATE TABLE serials_archive ( "
+		"serial_id int(11) DEFAULT '0' NOT NULL, "
+		"object_id int(10) unsigned DEFAULT '0' NOT NULL, "
+		"sequence_id int(10) unsigned DEFAULT '0' NOT NULL, "
+		"operation tinyint(4) unsigned DEFAULT '0' NOT NULL, "
+		"PRIMARY KEY (serial_id) "
+		") ");
   create_table(cmd);
 
   return(0);
@@ -1087,6 +1106,10 @@ int create_table (const char *cmd)
 
   int state;
 
+  if (Test) {
+      return 0;
+  }
+
   state = SQ_execute_query(connection, cmd, NULL);
   if (state != 0)
     {
@@ -1125,9 +1148,9 @@ int update_history_archive_sequence_id_and_timestamp (long ser_id, long new_seq_
 
   char cmd[MAXCMDLEN];
 
-  sprintf (cmd,"UPDATE history_archive
-		SET timestamp = %ld, sequence_id = %ld
-		WHERE serial = %ld",
+  sprintf (cmd,"UPDATE history_archive "
+	       "SET timestamp = %ld, sequence_id = %ld "
+	       "WHERE serial = %ld ",
 	   new_timestamp, new_seq_id, ser_id);
 
   return (execute_sql_command(cmd));
@@ -1147,9 +1170,9 @@ int update_history_archive_object_id_and_sequence_id (long ser_id, long new_obj_
 
   char cmd[MAXCMDLEN];
 
-  sprintf (cmd,"UPDATE history_archive
-		SET object_id = %ld, sequence_id = %ld
-		WHERE serial = %ld",
+  sprintf (cmd,"UPDATE history_archive "
+	       "SET object_id = %ld, sequence_id = %ld "
+	       "WHERE serial = %ld ",
 	   new_obj_id, new_seq_id, ser_id);
 
   return (execute_sql_command(cmd));
@@ -1169,10 +1192,10 @@ int update_prev_serial_of_object (long obj_id, long seq_id, long prev_ser, const
 
   char cmd[MAXCMDLEN];
 
-  sprintf (cmd,"UPDATE %s 
-		SET prev_serial = %ld 
-		WHERE object_id = %ld
-		AND sequence_id = %ld",
+  sprintf (cmd,"UPDATE %s "
+	       "SET prev_serial = %ld "
+	       "WHERE object_id = %ld "
+	       "AND sequence_id = %ld ",
 	   tablename, prev_ser, obj_id, seq_id);
 
   return(execute_sql_command(cmd));
@@ -1192,10 +1215,10 @@ int update_serial_of_object (long obj_id, long seq_id, long ser_id, const char *
 
   char cmd[MAXCMDLEN];
 
-  sprintf (cmd,"UPDATE %s 
-		SET serial = %ld 
-		WHERE object_id = %ld
-		AND sequence_id = %ld",
+  sprintf (cmd,"UPDATE %s "
+	       "SET serial = %ld "
+	       "WHERE object_id = %ld "
+	       "AND sequence_id = %ld ",
 	   tablename, ser_id, obj_id, seq_id);
 
   return(execute_sql_command(cmd));
@@ -1221,8 +1244,9 @@ int archive_serial (long ser_id, long obj_id, long seq_id, int op)
 
   char cmd[MAXCMDLEN];
 
-  sprintf (cmd, "INSERT INTO serials_archive (serial_id, object_id, sequence_id, operation)
-		 VALUES (%ld, %ld, %ld, %d)",
+  sprintf (cmd, "INSERT INTO serials_archive "
+                 "(serial_id, object_id, sequence_id, operation) "
+		  " VALUES (%ld, %ld, %ld, %d) ",
 	   ser_id, obj_id, seq_id, op);
 
   return (execute_sql_command(cmd));
@@ -1242,10 +1266,12 @@ int archive_failed_transaction (long ser_id)
 
   char cmd[MAXCMDLEN];
   
-  sprintf (cmd,"INSERT INTO history_archive (serial, timestamp, object)
-		SELECT failed_transaction.serial_id, failed_transaction.timestamp, failed_transaction.object
-		FROM failed_transaction
-		WHERE failed_transaction.serial_id = %ld",
+  sprintf (cmd,"INSERT INTO history_archive (serial, timestamp, object) "
+		"SELECT failed_transaction.serial_id, "
+		"       failed_transaction.timestamp, "
+		"	failed_transaction.object "
+		"FROM failed_transaction "
+		"WHERE failed_transaction.serial_id = %ld ",
 	   ser_id);
 
   return (execute_sql_command(cmd));
@@ -1266,13 +1292,18 @@ int copy_into_history_archive (long ser_id, long obj_id, long seq_id, const char
 
   char cmd[MAXCMDLEN];
   
-  sprintf (cmd,"INSERT INTO history_archive (object_id, sequence_id, serial, prev_serial, timestamp, object_type, object, pkey)
-		SELECT serials.object_id, serials.sequence_id, serials.serial_id, %s.prev_serial, %s.timestamp, %s.object_type, %s.object, %s.pkey
-		FROM serials,%s 
-		WHERE serials.serial_id = %ld 
-		AND %s.object_id = %ld 
-		AND %s.sequence_id = %ld",
-	   tablename, tablename, tablename, tablename, tablename, tablename, ser_id, tablename, obj_id, tablename, seq_id);
+  sprintf (cmd,"INSERT INTO history_archive "
+               "(object_id, sequence_id, serial, prev_serial, timestamp, "
+	       " object_type, object, pkey) "
+	       "SELECT serials.object_id, serials.sequence_id, "
+	       " serials.serial_id, %s.prev_serial, %s.timestamp, "
+	       " %s.object_type, %s.object, %s.pkey "
+	       "FROM serials,%s "
+	       "WHERE serials.serial_id = %ld "
+	       "AND %s.object_id = %ld "
+	       "AND %s.sequence_id = %ld ",
+	   tablename, tablename, tablename, tablename, tablename, 
+	   tablename, ser_id, tablename, obj_id, tablename, seq_id);
 
   return (execute_sql_command(cmd));
 
@@ -1295,16 +1326,23 @@ int copy_deleted_object_into_history_archive (long ser_id, long obj_id, long seq
   char cmd[MAXCMDLEN];
   int affected_rows;
 
-  sprintf (cmd,"INSERT INTO history_archive (object_id, sequence_id, serial, prev_serial, timestamp, object_type, object, pkey)
-		SELECT serials.object_id, serials.sequence_id, serials.serial_id, %s.serial, %s.timestamp, %s.object_type, %s.object, %s.pkey
-		FROM serials,%s 
-		WHERE serials.serial_id = %ld 
-		AND %s.object_id = %ld 
-		AND %s.sequence_id = %ld",
-	   tablename, tablename, tablename, tablename, tablename, tablename, ser_id, tablename, obj_id, tablename, seq_id);
+  sprintf (cmd,"INSERT INTO history_archive "
+               "(object_id, sequence_id, serial, prev_serial, timestamp, "
+	       " object_type, object, pkey) "
+	       "SELECT serials.object_id, serials.sequence_id, "
+	       " serials.serial_id, %s.serial, %s.timestamp, %s.object_type, "
+	       " %s.object, %s.pkey "
+	       "FROM serials,%s "
+ 	       "WHERE serials.serial_id = %ld "
+	       "AND %s.object_id = %ld "
+	       "AND %s.sequence_id = %ld ",
+	   tablename, tablename, tablename, tablename, tablename, tablename, 
+	   ser_id, tablename, obj_id, tablename, seq_id);
 
   affected_rows = execute_sql_command(cmd);
-  if (debug) printf ("copy_deleted_object_into_history_archive (%ld, %ld, %ld, %s): affected rows %d\n",ser_id,obj_id,seq_id,tablename,affected_rows);
+  if (debug) printf ("copy_deleted_object_into_history_archive "
+                     "(%ld, %ld, %ld, %s): affected rows %d\n",
+		     ser_id,obj_id,seq_id,tablename,affected_rows);
   return (affected_rows);
 
 } /* copy_deleted_object_into_history_archive() */
@@ -1326,7 +1364,7 @@ int delete_serial_entry (long ser_id)
 
   char cmd[MAXCMDLEN];
 
-  sprintf (cmd, "DELETE FROM serials WHERE serial_id = %ld", ser_id);
+  sprintf (cmd, "DELETE FROM serials WHERE serial_id = %ld ", ser_id);
 
   return (execute_sql_command(cmd));
 
@@ -1345,7 +1383,7 @@ int delete_failed_transaction_entry (long ser_id)
 
   char cmd[MAXCMDLEN];
 
-  sprintf (cmd, "DELETE FROM failed_transaction WHERE serial_id = %ld",ser_id);
+  sprintf (cmd, "DELETE FROM failed_transaction WHERE serial_id = %ld ",ser_id);
 
   return (execute_sql_command(cmd));
 
@@ -1368,7 +1406,7 @@ int delete_entry_from_object_table (long obj_id, long seq_id, const char* tablen
 
   if (debug) printf ("Deleting %s entry. Objid: %ld, seqid: %ld\n",tablename, obj_id, seq_id);
 
-  sprintf (cmd, "DELETE FROM %s WHERE object_id = %ld AND sequence_id = %ld",
+  sprintf (cmd, "DELETE FROM %s WHERE object_id = %ld AND sequence_id = %ld ",
 	   tablename, obj_id, seq_id);
 
   affected_rows = execute_sql_command(cmd);
@@ -1407,7 +1445,7 @@ int delete_serial_archive_entry (long ser_id)
 
   char cmd[MAXCMDLEN];
 
-  sprintf (cmd, "DELETE FROM serials_archive WHERE serial_id = %ld", ser_id);
+  sprintf (cmd, "DELETE FROM serials_archive WHERE serial_id = %ld ", ser_id);
 
   return (execute_sql_command(cmd));
 
@@ -1426,7 +1464,7 @@ int delete_history_archive_entry (long ser_id)
 
   char cmd[MAXCMDLEN];
 
-  sprintf (cmd, "DELETE FROM history_archive WHERE serial = %ld", ser_id);
+  sprintf (cmd, "DELETE FROM history_archive WHERE serial = %ld ", ser_id);
 
   return (execute_sql_command(cmd));
 
@@ -1470,12 +1508,13 @@ tblObjList *find_unreferenced_history_entries(long date)
   /* XXX Bug! This will find (and then remove) objects that would be
    * needed in the future by deletions. */
 
-  sprintf (query, "SELECT history.object_id, history.sequence_id, history.serial
-			FROM history LEFT JOIN serials
-			ON history.serial = serials.serial_id
-			WHERE serials.serial_id is NULL
-			AND history.serial != 0
-			AND history.timestamp < %ld", date);
+  sprintf (query, 
+      "SELECT history.object_id, history.sequence_id, history.serial "
+			"FROM history LEFT JOIN serials "
+			"ON history.serial = serials.serial_id "
+			"WHERE serials.serial_id is NULL "
+			"AND history.serial != 0 "
+			"AND history.timestamp < %ld ", date);
   execute_sql_query(query, &result);
 
   /* Foreach entry: */
@@ -1617,7 +1656,7 @@ int delete_dummy_history_objects()
   char cmd[MAXCMDLEN];
   int rc;
 
-  sprintf (cmd,"DELETE FROM history WHERE object_type = %d",OBJ_TYPE_DUMMY);
+  sprintf (cmd,"DELETE FROM history WHERE object_type = %d ",OBJ_TYPE_DUMMY);
 
   lock_last_history_serial_tables();
   rc = execute_sql_command(cmd);
@@ -1737,6 +1776,10 @@ int execute_sql_command (const char *cmd)
 
   int state;
 
+  if (Test) {
+      return 0;
+  }
+
   state = SQ_execute_query(connection, cmd, NULL);
   if (state != 0)
     {
@@ -1769,14 +1812,14 @@ int create_auxiliary_table()
   int state;
   char cmd[MAXCMDLEN];
 
-  sprintf (cmd,"CREATE TABLE hs_auxiliary (
-		serial int(11) DEFAULT '0' NOT NULL,
-		object_id int(10) unsigned DEFAULT '0' NOT NULL,
-		sequence_id int(10) unsigned DEFAULT '0' NOT NULL,
-		atlast tinyint(4) unsigned DEFAULT '0' NOT NULL,
-		checkpoint tinyint(4) unsigned DEFAULT '0' NOT NULL,
-		PRIMARY KEY (serial)
-		);");
+  sprintf (cmd,"CREATE TABLE hs_auxiliary ( "
+		"serial int(11) DEFAULT '0' NOT NULL, "
+		"object_id int(10) unsigned DEFAULT '0' NOT NULL, "
+		"sequence_id int(10) unsigned DEFAULT '0' NOT NULL, "
+		"atlast tinyint(4) unsigned DEFAULT '0' NOT NULL, "
+		"checkpoint tinyint(4) unsigned DEFAULT '0' NOT NULL, "
+		"PRIMARY KEY (serial) "
+		") ");
   state = create_table(cmd);
   if (state == 0) /* state != 0 only if the table already exists - other errors make the program die */
     {
@@ -1803,8 +1846,9 @@ int reset_auxiliary_table()
 
   char cmd[MAXCMDLEN];
 
-  sprintf (cmd,"UPDATE hs_auxiliary SET
-		serial = 0, object_id = 0, sequence_id = 0, atlast = 0, checkpoint = 0");
+  sprintf (cmd,"UPDATE hs_auxiliary SET "
+		"serial = 0, object_id = 0, sequence_id = 0, "
+		"atlast = 0, checkpoint = 0 ");
 
   return(execute_sql_command(cmd));
 
@@ -1823,7 +1867,7 @@ int drop_auxiliary_table()
 
   char cmd[MAXCMDLEN];
 
-  sprintf (cmd,"DROP TABLE IF EXISTS hs_auxiliary");
+  sprintf (cmd,"DROP TABLE IF EXISTS hs_auxiliary ");
   return(execute_sql_command(cmd));
  
 } /* drop_auxiliary_table() */
@@ -1841,12 +1885,12 @@ int update_hs_auxiliary_checkpoint(long ser_id, long obj_id, long seq_id, int at
 
   char cmd[MAXCMDLEN];
 
-  sprintf (cmd,"UPDATE hs_auxiliary 
-		SET serial = %ld,
-		object_id = %ld,
-		sequence_id = %ld,
-		atlast = %d,
-		checkpoint = %d",
+  sprintf (cmd,"UPDATE hs_auxiliary "
+		"SET serial = %ld, "
+		"object_id = %ld, "
+		"sequence_id = %ld, "
+		"atlast = %d, "
+		"checkpoint = %d ",
 	   ser_id, obj_id, seq_id, atlast, checkpoint);
 
   return (execute_sql_command(cmd));
@@ -1871,7 +1915,6 @@ int crash_recovery()
   long serial, objid, seqid;
   int atlast, chkp;
 
- 
   if (!exists_checkpointing_table())
     {
       /* The checkpointing table does not exist, there was no crash!
